#include "Adabinding.h"

//Message

MessageC::MessageC(void){
}

void MessageC::SetMessage(int p, const char* rK, int eT, const char* dM, const char* cT, const char* cE, const char* c){
	priority = p;
	routingKey = rK;
	expirationTime = eT;
	deliveryMode = dM;
	contentType = cT;
	contentEncoding = cE;
	content = c;
}

const char* MessageC::GetContent(){
	return content;
}
/****************************/

//Session
SessionC::SessionC(void){
}

void SessionC::MessageTransfer(const char* destination, MessageC & msg){
	Message message;
	message.setData(msg.content);
	message.getDeliveryProperties().setPriority(msg.priority);
	message.getDeliveryProperties().setRoutingKey(msg.routingKey);
	message.getMessageProperties().setContentEncoding(msg.contentEncoding);
	message.getMessageProperties().setContentType(msg.contentType);
	if(msg.expirationTime > 0)
		message.getDeliveryProperties().setTtl(msg.expirationTime);
	
	this->session.messageTransfer(arg::content=message,  arg::destination=destination, arg::acceptMode=0);
}

void SessionC::Bind(const char* queueName, const char* exchangeName, const char* bindingKey){
	session.sync();
	session.exchangeBind(arg::exchange=exchangeName, arg::queue=queueName, arg::bindingKey=bindingKey);
	session.sync();
}

void SessionC::ExchangeDeclare(const char* exchangeName, const char* exchangeType, const char* alternateExchange, bool durable, bool autoDelete){
	session.exchangeDeclare(exchangeName, exchangeType, alternateExchange, false, durable, autoDelete);	
}
					
void SessionC::Unbind(const char* queueName, const char* exchangeName, const char* routingKey){
	session.exchangeUnbind(queueName, exchangeName, routingKey);
}

void SessionC::QueueDeclare(const char* queueName, const char* alternateExchange, bool durable, bool exclusive, bool autoDelete){
	session.queueDeclare(queueName, alternateExchange, false, durable, exclusive, autoDelete);
}

void SessionC::QueueDelete(const char* queueName, bool if_Unused){
	session.queueDelete(queueName, if_Unused);
}

void SessionC::Purge(const char* queueName){
	session.queuePurge(queueName);
}

void SessionC::ExchangeDelete(const char* exchangeName, bool if_Unused){
	session.exchangeDelete(exchangeName, if_Unused);
}
    					
 
/****************************/

//Connection

ConnectionC::ConnectionC(){
}

ConnectionC::~ConnectionC(){
	free(connection);
}

void ConnectionC::Open(const char* host, int port, const char* uid, const char* pwd){
	connection = new Connection();
	connection->open(host, port);
}

void ConnectionC::Close(){
	connection->close();
}

void ConnectionC::StartSession(SessionC ses) const{
	ses.session = connection->newSession();
}
/****************************/

//LocalQueue
LocalQueueC::LocalQueueC(void){
	localQueue = new LocalQueue();
}

LocalQueueC::~LocalQueueC(){
	cout << "Destruyendo LocalQueue" << endl;
	free(localQueue);
}

bool LocalQueueC::Empty() const{
	return localQueue->empty();
}

int LocalQueueC::Size() const{
	return localQueue->size();
}

void LocalQueueC::GetNoSub(char* queueName, SessionC ses, MessageC &msgC, int timeOut){
	
	SubscriptionManager subscriptions(ses.session);
 	subscriptions.subscribe(*localQueue, "adaqueue");
 	Message msg;
 	if(timeOut > 0)
 		localQueue->get(msg, timeOut*TIME_SEC);
 	else
 		msg = localQueue->get();
	int priority = msg.getDeliveryProperties().getPriority();
	const char* routingKey = msg.getDeliveryProperties().getRoutingKey().c_str();
	int expirationTime = 0;
	const char* deliveryMode = "";
	const char* contentType = msg.getMessageProperties().getContentType().c_str();
	const char* contentEncoding = msg.getMessageProperties().getContentEncoding().c_str();
	const char* content = msg.getData().c_str();
	msgC.SetMessage(priority, routingKey, expirationTime, deliveryMode, contentType, contentEncoding, content);
}
